{% load i18n %}
{% load navactive %}

<li class="nav-item">
    <a class="nav-link" href="{% url 'squads:groups' %}">Squads</a>
</li>

{% if perms.squads.admin_access or perms.squads.squad_manager %}
<li class="nav-item">
    <a class="nav-link" href="{% url 'squads:create_group' %}">Create Squad</a>
</li>

<li class="nav-item">
    <a class="nav-link" href="{% url 'squads:manage_pendings' %}">Manage Pendings</a>
</li>

<li class="nav-item">
    <a class="nav-link" href="{% url 'squads:manage_members' %}">Manage Members</a>
</li>

<li class="nav-item">
    <a class="nav-link" href="{% url 'squads:manage_groups' %}">Manage Squads</a>
</li>
{% endif %}
