# Generated by Django 4.2.11 on 2024-06-11 20:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("squads", "0001_initial"),
    ]

    operations = [
        migrations.RenameField(
            model_name="groups",
            old_name="require_approval",
            new_name="req_approve",
        ),
        migrations.RenameField(
            model_name="memberships",
            old_name="has_required_skills",
            new_name="req_filters",
        ),
        migrations.RenameField(
            model_name="pending",
            old_name="application",
            new_name="comment",
        ),
        migrations.AddField(
            model_name="pending",
            name="req_filters",
            field=models.BooleanField(default=True),
        ),
    ]
